@@ -25,7 +25,7 @@ static void explain(void)
 	fprintf(stderr, "       tunnel_key set <TUNNEL_KEY>\n");
 	fprintf(stderr,
 		"Where TUNNEL_KEY is a combination of:\n"
-		"id <TUNNELID> (mandatory)\n"
+		"id <TUNNELID>\n"
 		"src_ip <IP> (mandatory)\n"
 		"dst_ip <IP> (mandatory)\n"
 		"dst_port <UDP_PORT>\n"
@@ -217,7 +217,6 @@ static int parse_tunnel_key(struct action_util *a, int *argc_p, char ***argv_p,
 	int ret;
 	int has_src_ip = 0;
 	int has_dst_ip = 0;
-	int has_key_id = 0;
 	int csum = 1;
 
 	if (matches(*argv, "tunnel_key") != 0)
@@ -273,7 +272,6 @@ static int parse_tunnel_key(struct action_util *a, int *argc_p, char ***argv_p,
 				fprintf(stderr, "Illegal \"id\"\n");
 				return -1;
 			}
-			has_key_id = 1;
 		} else if (matches(*argv, "dst_port") == 0) {
 			NEXT_ARG();
 			ret = tunnel_key_parse_dst_port(*argv,
@@ -335,7 +333,7 @@ static int parse_tunnel_key(struct action_util *a, int *argc_p, char ***argv_p,
 	}
 
 	if (action == TCA_TUNNEL_KEY_ACT_SET &&
-	    (!has_src_ip || !has_dst_ip || !has_key_id)) {
+	    (!has_src_ip || !has_dst_ip)) {
 		fprintf(stderr, "set needs tunnel_key parameters\n");
 		explain();
 		return -1;
